function iterpolate(x, y, z)
	return (z - x) / (y - x)
end


function sine()
	for i=1, SAMPLES do
		BASE_WAVEFORM[i] = math.sin(i * math.pi / SAMPLES * 2)
	end
end


function triangle()
	local volt = 0
	local ctvrtina_samplu = SAMPLES / 4
	local ctvrtina2 = ctvrtina_samplu * 2
	local ctvrtina3 = ctvrtina_samplu * 3
	local delenec = 1 / SAMPLES * 4
	
	for i=1, SAMPLES do
		
		if i < ctvrtina_samplu then
			volt = volt + delenec
		end
		
		if i >= ctvrtina_samplu and i < ctvrtina2 then
			volt = volt - delenec
		end

		if i >= ctvrtina2 and i < ctvrtina3 then
			volt = volt - delenec
		end
		
		if i >= ctvrtina3 and i < SAMPLES then
			volt = volt + delenec
		end

		BASE_WAVEFORM[i] = volt
	end
end


function saw()
	local volt = -1
	local result = 1 / SAMPLES * 4

	for i=1, SAMPLES do
		volt = volt + result
		if volt > 1 then volt = -1 end
		BASE_WAVEFORM[i] = volt
	end
end


function square()
	local volt = 1
	local result = SAMPLES / 2

	for i=1, SAMPLES do
		if i == result then
			volt = -1
		end
		BASE_WAVEFORM[i] = volt
	end
end


function neuron(h, i)
	local n=1
	local x0, y0, z0 = 0, 0, 0
	local xval, yval, zval = 0, 0, 0

	for n=1, SAMPLES do
		xval = x0 + h*(y0 - (x0^3) + 3*(x0^2) - z0 + i)
		yval = y0 + h * (1 - 5*(x0^2) - y0)
		zval = z0 + h * (0.01* ((4*(x0 + 1.6)) - z0))
		x0 = xval
		y0 = yval
		z0 = zval
		BASE_WAVEFORM[n] = xval
		--USER_WAVEFORM[n] = xval
	end
end


function wave_calc(wave_no)

	local out = 0
	local abs = math.abs

	local i
	for i=1, 256 do
		local pos = out + BASE_WAVEFORM[((wave_no * i ) % SAMPLES)+1] * HARMONIC_SERIES[i]
		out = pos
	end

	local normalize = 0
	for i=1, #HARMONIC_SERIES do
		normalize = normalize + abs(HARMONIC_SERIES[i])
	end

	return out / normalize
end


function phase_distortion(buffer, freq, x1, y1, x2)
	local delta = freq / 44100 --frequency / samplerate
	local pos = 0
	local xDelta = (x2 - x1) / #buffer
	local warpedPos = nil
	local m = nil
	local b = nil
	local x = x1
	local pi = math.pi
	local cos = math.cos
	local sin = math.sin

	for i = 1, #buffer do
		
		if pos < x then
			m = y1 / x
			warpedPos = m * pos
		else
			m = (1.0 - y1) / (1.0 - x)
			b = 1.0 - m
			warpedPos = m * pos + b
		end

		buffer[i] = cos(2.0 * pi * warpedPos)
		pos = pos + delta

		while pos >= 1.0 do
			pos = pos - 1.0
		end

		x = x + xDelta

	end

end

sine()