COLUMN_SPACING = 2
COLUMN_MARGIN = 2
FIRST_RUN = true

function prepare_for_start()
  if (dialog and dialog.visible) then
    dialog:show()
    return
  
  else

    dialog = renoise.app():show_custom_dialog("Harmoniks by martblek", make_gui)
  
    if FIRST_RUN == false then
      return
    end

    for page=0, 225, 32 do
      for i=page+1, page + 32 do
        vbs["harmony"..page]:add_child(
          vb:column{
            width=12,
            --margin=4,
            --spacing=4,
            vb:minislider{
              width=12,
              id = "Harmony" .. i,
              height=120,
              min=-1,
              max=1,
              value=HARMONIC_SERIES[i],
              notifier=function(value)
                vbs["Harmony" .. i].value = value
                HARMONIC_SERIES[i] = value
                if vbs["Harmony"..i].value > 0 or vbs["Harmony"..i].value < 0 then
                  vbs["Harmony_btn"..i].color = {0,math.abs(math.floor(255 * value)),0}
                else
                  vbs["Harmony_btn"..i].color = {0,0,0}
                end
                if vbs.phase_on_off.value == true then
                  phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, vbs.y1.value, vbs.x2.value)
                end
                draw_sample()
              end
            },
            vb:button{
              id="Harmony_btn"..i,
              width=12,
              height=12,
              color={0, 0, 0},
              notifier = function(bool)
                vbs["Harmony_btn"..i].color = {0, 0, 0}
                vbs["Harmony"..i].value = 0
                if vbs.phase_on_off.value == true then
                  phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, vbs.y1.value, vbs.x2.value)
                end
                draw_sample()
              end
            },
        })
      end
    end

    vbs.harmony0.visible=true
    vbs.harmony32.visible=false
    vbs.harmony64.visible=false
    vbs.harmony96.visible=false
    vbs.harmony128.visible=false
    vbs.harmony160.visible=false
    vbs.harmony192.visible=false
    vbs.harmony224.visible=false
  end
  FIRST_RUN = false
end

function create_selected_wave_type(wave_type)
  if wave_type == 1 then
    sine()
  elseif wave_type == 2 then
    triangle()
  elseif wave_type == 3 then
    saw()
  elseif wave_type == 4 then
    square()
  elseif wave_type == 5 then
    neuron(3, 2)
  end
end

function reset_harmonies()
  for i=1, 256 do
    HARMONIC_SERIES[i] = 0
    vb.views["Harmony"..i].value = 0
  end
  HARMONIC_SERIES[1] = 1
  vb.views["Harmony1"].value = 1
  -- reset phase_distortion
  vbs.freq.value = 1
  vbs.x1.value = 1
  vbs.x2.value = 1
  vbs.y1.value = 1
end

function rotate_inplace(t, d, n)

  local function reverse(t, i, j)
    while i < j do
      t[i], t[j] = t[j], t[i]
      i, j = i + 1, j - 1
    end
  end

  n = n or #t
  d = (d or 1) % n
  reverse(t, 1, n)
  reverse(t, 1, d)
  reverse(t, d + 1, n)
end


make_gui =
    vb:column{
      style="group",
      margin = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN,
      spacing = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING,
      height = renoise.ViewBuilder.DEFAULT_CONTROL_HEIGHT,
      vb:horizontal_aligner{
        mode="distribute",
        spacing=4,
        vb:popup{
          id="buffers",
          items=BUFFERS,
          value=1,
          notifier = function(len)
            SAMPLES = tonumber(BUFFERS[len])
            BASE_WAVEFORM = {}
            local wav_type = vbs["wave_selector"].value
            create_selected_wave_type(wav_type)
            if vbs.phase_on_off.value == true then
              phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, vbs.y1.value, vbs.x2.value)
            end
            draw_sample()
          end  
        },
        vb:popup{
          id="wave_selector",
          items={"Sine", "Triangle", "Saw", "Square"}, -- "Neuron"},
          value=1,
          notifier=function(wave_type)
            create_selected_wave_type(wave_type)
            if vbs.phase_on_off.value == true then
              phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, vbs.y1.value, vbs.x2.value)
            end
            draw_sample()
          end
        },
        vb:button{
          text="Reset Harmonies",
          notifier=function()
            reset_harmonies()
            draw_sample()
          end
        },
        vb:button{
          text="Make actual as Base",
          notifier=function()
            for i=1, SAMPLES-1 do
              BASE_WAVEFORM[i] = USER_WAVEFORM[i]
            end
            vbs.phase_on_off.value = false
            rotate_inplace(BASE_WAVEFORM)
            reset_harmonies()
            draw_sample()
          end
        },
      },
      vb:space{height=5},
      vb:column{
        style="group",
        spacing=4,
        margin=4,
        vb:horizontal_aligner{
          mode="distribute",
          vb:checkbox{
            id="phase_on_off",
            value=false,
            notifier = function(bool)
              if bool == true then
                phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, vbs.y1.value, vbs.x2.value)
              end
              draw_sample()
            end
          },
          vb:row{
            vb:slider{
              id="x1",
              width=100,
              min=0.0,
              max=10.0,
              value=1,
              steps={0.01, 0.1},
              notifier = function(value)
                if vbs.phase_on_off.value == false then
                  return
                end
                phase_distortion(BASE_WAVEFORM, vbs.freq.value, value, vbs.y1.value, vbs.x2.value)
                draw_sample()
              end
            },
            vb:slider{
              id="y1",
              width=100,
              min=0.0,
              max=10.0,
              value=1,
              steps={0.01, 0.1},
              notifier = function(value)
                if vbs.phase_on_off.value == false then
                  return
                end
                phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, value, vbs.x2.value)
                draw_sample()
              end
            },
            vb:slider{
              id="x2",
              width=100,
              min=0.0,
              max=10.0,
              value=1,
              steps={0.01, 0.1},
              notifier = function(value)
                if vbs.phase_on_off.value == false then
                  return
                end
                phase_distortion(BASE_WAVEFORM, vbs.freq.value, vbs.x1.value, vbs.y1.value, value)
                draw_sample()
              end
            },
            vb:slider{
              id="freq",
              width=100,
              min=0.0,
              max=1000.0,
              value=1,
              steps={0.1, 1},
              notifier = function(value)
                if vbs.phase_on_off.value == false then
                  return
                end
                phase_distortion(BASE_WAVEFORM, value, vbs.x1.value, vbs.y1.value, vbs.x2.value)
                draw_sample()
              end
            },
          },
        },
        vb:horizontal_aligner{
          mode="distribute",
          vb:switch{
            width=400,
            height=22,
            items={"1-32", "33-64", "65-96", "97-128", "129-160", "161-192", "193-224", "225-256"},
            value=HARMONY_OFFSET,
            notifier=function(page)
              vbs.harmony0.visible=false
              vbs.harmony32.visible=false
              vbs.harmony64.visible=false
              vbs.harmony96.visible=false
              vbs.harmony128.visible=false
              vbs.harmony160.visible=false
              vbs.harmony192.visible=false
              vbs.harmony224.visible=false
            
              vb.views["harmony"..(page-1)*32].visible = true
            end
          },
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony0",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony32",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony64",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony96",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony128",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony160",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony192",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
        vb:horizontal_aligner{
          mode="distribute",
          id="harmony224",
          spacing=COLUMN_SPACING,
          margin = COLUMN_MARGIN,
        },
      },
    }